package Crypt::AuthEnc;

use strict;
use warnings;
our $VERSION = '0.088_001';

### not used

1;

=pod

=head1 NAME

Crypt::AuthEnc - [internal only]

=head1 SYNOPSIS

Do not use this module directly.

Use a concrete authenticated-encryption module such as L<Crypt::AuthEnc::GCM>
or L<Crypt::AuthEnc::ChaCha20Poly1305>.

=head1 DESCRIPTION

Internal namespace placeholder for authenticated-encryption modules.

Do not use this module directly. Use a concrete implementation such as
L<Crypt::AuthEnc::ChaCha20Poly1305>, L<Crypt::AuthEnc::XChaCha20Poly1305>,
L<Crypt::AuthEnc::EAX>, L<Crypt::AuthEnc::GCM>, or L<Crypt::AuthEnc::OCB>.

=cut
